package de.joethei.hs.java2.praktikum.praktikum4;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class KlausurenServer {

    private Map<String, Set<Integer>> data = new HashMap<>();
    private static ExecutorService executorService = Executors.newCachedThreadPool();
    private static Map<String, CommandHandler> commands = new HashMap<>();

    public KlausurenServer(int port) {
        commands.put("put", (writer, args) -> {
           if(args.size() >= 2) {
               Set<Integer> set = new HashSet<>();
               for(int i = 1; i <= args.size(); i++) {
                   try {
                       set.add(Integer.parseInt(args.get(i)));
                   }catch (NumberFormatException ex) {
                       writer.write("0");
                   }
               }
               data.put(args.get(0), set);
           }else {
               writer.write("0");
           }
        });
        commands.put("getall", (writer, args) -> {
           if(!data.isEmpty()) {
               writer.write("1 ");
               Set<Integer> set = new TreeSet<>();
               data.forEach((s, integers) -> set.addAll(integers));
               writer.write(set.toString());
           }else {
               writer.write("0");
           }
        });
        commands.put("stop", ((writer, args) -> {
            writer.write("1");
            executorService.shutdown();
            System.exit(0);
        }));

        Runnable runnable = () -> {
            try {
                ServerSocket serverSocket = new ServerSocket(port);
                System.out.println("waiting for clients");
                while (true) {
                    Socket clientSocket = serverSocket.accept();
                    executorService.submit(new ClientRunnable(clientSocket));
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

        };

        Thread thread = new Thread(runnable);
        thread.start();
    }

    static Map<String, CommandHandler> getCommands() {
        return commands;
    }

    public static void main(String[] args) {
        new KlausurenServer(Integer.parseInt(args[0]));
    }
}